diff --strip-trailing-cr -Nur filezilla-3.7.3.orig/src/engine/proxy.cpp filezilla-3.7.3/src/engine/proxy.cpp
--- filezilla-3.7.3.orig/src/engine/proxy.cpp	2012-11-02 12:30:06.000000000 +0800
+++ filezilla-3.7.3/src/engine/proxy.cpp	2013-10-01 12:03:16.806583500 +0800
@@ -1,6 +1,7 @@
 #include <filezilla.h>
 #include "proxy.h"
 #include <errno.h>
+#include <wx/tokenzr.h>
 #include "ControlSocket.h"
 
 enum handshake_state
@@ -25,6 +26,8 @@
 	m_pSendBuffer = 0;
 	m_pRecvBuffer = 0;
 
+	m_doAuth = 0;
+
 	m_proxyType = unknown;
 
 	m_can_write = false;
@@ -39,6 +42,342 @@
 	delete [] m_pRecvBuffer;
 }
 
+static void AppendSubHex(unsigned char val, wxString& result)
+{
+   if (val > 9)
+   {
+      result.Append((char)('a'+val-10));
+   }
+   else
+   {
+      result.Append((char)('0'+val));
+   }
+}
+
+static void AppendHex(unsigned char val, wxString& result)
+{
+   AppendSubHex((val >> 4) & 0xF, result);
+   AppendSubHex(val&0xF, result);
+}
+
+/*
+ * This is an OpenSSL-compatible implementation of the RSA Data Security, Inc.
+ * MD5 Message-Digest Algorithm (RFC 1321).
+ *
+ * Homepage:
+ * http://openwall.info/wiki/people/solar/software/public-domain-source-code/md5
+ *
+ * Author:
+ * Alexander Peslyak, better known as Solar Designer <solar at openwall.com>
+ *
+ * This software was written by Alexander Peslyak in 2001.  No copyright is
+ * claimed, and the software is hereby placed in the public domain.
+ * In case this attempt to disclaim copyright and place the software in the
+ * public domain is deemed null and void, then the software is
+ * Copyright (c) 2001 Alexander Peslyak and it is hereby released to the
+ * general public under the following terms:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
+ *
+ * (This is a heavily cut-down "BSD license".)
+ *
+ * This differs from Colin Plumb's older public domain implementation in that
+ * no exactly 32-bit integer data type is required (any 32-bit or wider
+ * unsigned integer data type will do), there's no compile-time endianness
+ * configuration, and the function prototypes match OpenSSL's.  No code from
+ * Colin Plumb's implementation has been reused; this comment merely compares
+ * the properties of the two independent implementations.
+ *
+ * The primary goals of this implementation are portability and ease of use.
+ * It is meant to be fast, but not as fast as possible.  Some known
+ * optimizations are not included to reduce source code size and avoid
+ * compile-time configuration.
+ */
+
+#include <string.h>
+
+/* Any 32-bit or wider unsigned integer data type will do */
+typedef unsigned int MD5_u32plus;
+
+typedef struct {
+	MD5_u32plus lo, hi;
+	MD5_u32plus a, b, c, d;
+	unsigned char buffer[64];
+	MD5_u32plus block[16];
+} MD5_CTX;
+
+/*
+ * The basic MD5 functions.
+ *
+ * F and G are optimized compared to their RFC 1321 definitions for
+ * architectures that lack an AND-NOT instruction, just like in Colin Plumb's
+ * implementation.
+ */
+#define F(x, y, z)			((z) ^ ((x) & ((y) ^ (z))))
+#define G(x, y, z)			((y) ^ ((z) & ((x) ^ (y))))
+#define H(x, y, z)			((x) ^ (y) ^ (z))
+#define I(x, y, z)			((y) ^ ((x) | ~(z)))
+
+/*
+ * The MD5 transformation for all four rounds.
+ */
+#define STEP(f, a, b, c, d, x, t, s) \
+	(a) += f((b), (c), (d)) + (x) + (t); \
+	(a) = (((a) << (s)) | (((a) & 0xffffffff) >> (32 - (s)))); \
+	(a) += (b);
+
+/*
+ * SET reads 4 input bytes in little-endian byte order and stores them
+ * in a properly aligned word in host byte order.
+ *
+ * The check for little-endian architectures that tolerate unaligned
+ * memory accesses is just an optimization.  Nothing will break if it
+ * doesn't work.
+ */
+#if defined(__i386__) || defined(__x86_64__) || defined(__vax__)
+#define SET(n) \
+	(*(MD5_u32plus *)&ptr[(n) * 4])
+#define GET(n) \
+	SET(n)
+#else
+#define SET(n) \
+	(ctx->block[(n)] = \
+	(MD5_u32plus)ptr[(n) * 4] | \
+	((MD5_u32plus)ptr[(n) * 4 + 1] << 8) | \
+	((MD5_u32plus)ptr[(n) * 4 + 2] << 16) | \
+	((MD5_u32plus)ptr[(n) * 4 + 3] << 24))
+#define GET(n) \
+	(ctx->block[(n)])
+#endif
+
+/*
+ * This processes one or more 64-byte data blocks, but does NOT update
+ * the bit counters.  There are no alignment requirements.
+ */
+static void *body(MD5_CTX *ctx, void *data, unsigned long size)
+{
+	unsigned char *ptr;
+	MD5_u32plus a, b, c, d;
+	MD5_u32plus saved_a, saved_b, saved_c, saved_d;
+
+	ptr = (unsigned char *)data;
+
+	a = ctx->a;
+	b = ctx->b;
+	c = ctx->c;
+	d = ctx->d;
+
+	do {
+		saved_a = a;
+		saved_b = b;
+		saved_c = c;
+		saved_d = d;
+
+/* Round 1 */
+		STEP(F, a, b, c, d, SET(0), 0xd76aa478, 7)
+		STEP(F, d, a, b, c, SET(1), 0xe8c7b756, 12)
+		STEP(F, c, d, a, b, SET(2), 0x242070db, 17)
+		STEP(F, b, c, d, a, SET(3), 0xc1bdceee, 22)
+		STEP(F, a, b, c, d, SET(4), 0xf57c0faf, 7)
+		STEP(F, d, a, b, c, SET(5), 0x4787c62a, 12)
+		STEP(F, c, d, a, b, SET(6), 0xa8304613, 17)
+		STEP(F, b, c, d, a, SET(7), 0xfd469501, 22)
+		STEP(F, a, b, c, d, SET(8), 0x698098d8, 7)
+		STEP(F, d, a, b, c, SET(9), 0x8b44f7af, 12)
+		STEP(F, c, d, a, b, SET(10), 0xffff5bb1, 17)
+		STEP(F, b, c, d, a, SET(11), 0x895cd7be, 22)
+		STEP(F, a, b, c, d, SET(12), 0x6b901122, 7)
+		STEP(F, d, a, b, c, SET(13), 0xfd987193, 12)
+		STEP(F, c, d, a, b, SET(14), 0xa679438e, 17)
+		STEP(F, b, c, d, a, SET(15), 0x49b40821, 22)
+
+/* Round 2 */
+		STEP(G, a, b, c, d, GET(1), 0xf61e2562, 5)
+		STEP(G, d, a, b, c, GET(6), 0xc040b340, 9)
+		STEP(G, c, d, a, b, GET(11), 0x265e5a51, 14)
+		STEP(G, b, c, d, a, GET(0), 0xe9b6c7aa, 20)
+		STEP(G, a, b, c, d, GET(5), 0xd62f105d, 5)
+		STEP(G, d, a, b, c, GET(10), 0x02441453, 9)
+		STEP(G, c, d, a, b, GET(15), 0xd8a1e681, 14)
+		STEP(G, b, c, d, a, GET(4), 0xe7d3fbc8, 20)
+		STEP(G, a, b, c, d, GET(9), 0x21e1cde6, 5)
+		STEP(G, d, a, b, c, GET(14), 0xc33707d6, 9)
+		STEP(G, c, d, a, b, GET(3), 0xf4d50d87, 14)
+		STEP(G, b, c, d, a, GET(8), 0x455a14ed, 20)
+		STEP(G, a, b, c, d, GET(13), 0xa9e3e905, 5)
+		STEP(G, d, a, b, c, GET(2), 0xfcefa3f8, 9)
+		STEP(G, c, d, a, b, GET(7), 0x676f02d9, 14)
+		STEP(G, b, c, d, a, GET(12), 0x8d2a4c8a, 20)
+
+/* Round 3 */
+		STEP(H, a, b, c, d, GET(5), 0xfffa3942, 4)
+		STEP(H, d, a, b, c, GET(8), 0x8771f681, 11)
+		STEP(H, c, d, a, b, GET(11), 0x6d9d6122, 16)
+		STEP(H, b, c, d, a, GET(14), 0xfde5380c, 23)
+		STEP(H, a, b, c, d, GET(1), 0xa4beea44, 4)
+		STEP(H, d, a, b, c, GET(4), 0x4bdecfa9, 11)
+		STEP(H, c, d, a, b, GET(7), 0xf6bb4b60, 16)
+		STEP(H, b, c, d, a, GET(10), 0xbebfbc70, 23)
+		STEP(H, a, b, c, d, GET(13), 0x289b7ec6, 4)
+		STEP(H, d, a, b, c, GET(0), 0xeaa127fa, 11)
+		STEP(H, c, d, a, b, GET(3), 0xd4ef3085, 16)
+		STEP(H, b, c, d, a, GET(6), 0x04881d05, 23)
+		STEP(H, a, b, c, d, GET(9), 0xd9d4d039, 4)
+		STEP(H, d, a, b, c, GET(12), 0xe6db99e5, 11)
+		STEP(H, c, d, a, b, GET(15), 0x1fa27cf8, 16)
+		STEP(H, b, c, d, a, GET(2), 0xc4ac5665, 23)
+
+/* Round 4 */
+		STEP(I, a, b, c, d, GET(0), 0xf4292244, 6)
+		STEP(I, d, a, b, c, GET(7), 0x432aff97, 10)
+		STEP(I, c, d, a, b, GET(14), 0xab9423a7, 15)
+		STEP(I, b, c, d, a, GET(5), 0xfc93a039, 21)
+		STEP(I, a, b, c, d, GET(12), 0x655b59c3, 6)
+		STEP(I, d, a, b, c, GET(3), 0x8f0ccc92, 10)
+		STEP(I, c, d, a, b, GET(10), 0xffeff47d, 15)
+		STEP(I, b, c, d, a, GET(1), 0x85845dd1, 21)
+		STEP(I, a, b, c, d, GET(8), 0x6fa87e4f, 6)
+		STEP(I, d, a, b, c, GET(15), 0xfe2ce6e0, 10)
+		STEP(I, c, d, a, b, GET(6), 0xa3014314, 15)
+		STEP(I, b, c, d, a, GET(13), 0x4e0811a1, 21)
+		STEP(I, a, b, c, d, GET(4), 0xf7537e82, 6)
+		STEP(I, d, a, b, c, GET(11), 0xbd3af235, 10)
+		STEP(I, c, d, a, b, GET(2), 0x2ad7d2bb, 15)
+		STEP(I, b, c, d, a, GET(9), 0xeb86d391, 21)
+
+		a += saved_a;
+		b += saved_b;
+		c += saved_c;
+		d += saved_d;
+
+		ptr += 64;
+	} while (size -= 64);
+
+	ctx->a = a;
+	ctx->b = b;
+	ctx->c = c;
+	ctx->d = d;
+
+	return ptr;
+}
+
+void MD5_Init(MD5_CTX *ctx)
+{
+	ctx->a = 0x67452301;
+	ctx->b = 0xefcdab89;
+	ctx->c = 0x98badcfe;
+	ctx->d = 0x10325476;
+
+	ctx->lo = 0;
+	ctx->hi = 0;
+}
+
+void MD5_Update(MD5_CTX *ctx, void *data, unsigned long size)
+{
+	MD5_u32plus saved_lo;
+	unsigned long used, free;
+
+	saved_lo = ctx->lo;
+	if ((ctx->lo = (saved_lo + size) & 0x1fffffff) < saved_lo)
+		ctx->hi++;
+	ctx->hi += size >> 29;
+
+	used = saved_lo & 0x3f;
+
+	if (used) {
+		free = 64 - used;
+
+		if (size < free) {
+			memcpy(&ctx->buffer[used], data, size);
+			return;
+		}
+
+		memcpy(&ctx->buffer[used], data, free);
+		data = (unsigned char *)data + free;
+		size -= free;
+		body(ctx, ctx->buffer, 64);
+	}
+
+	if (size >= 64) {
+		data = body(ctx, data, size & ~(unsigned long)0x3f);
+		size &= 0x3f;
+	}
+
+	memcpy(ctx->buffer, data, size);
+}
+
+void MD5_Final(unsigned char *result, MD5_CTX *ctx)
+{
+	unsigned long used, free;
+
+	used = ctx->lo & 0x3f;
+
+	ctx->buffer[used++] = 0x80;
+
+	free = 64 - used;
+
+	if (free < 8) {
+		memset(&ctx->buffer[used], 0, free);
+		body(ctx, ctx->buffer, 64);
+		used = 0;
+		free = 64;
+	}
+
+	memset(&ctx->buffer[used], 0, free - 8);
+
+	ctx->lo <<= 3;
+	ctx->buffer[56] = ctx->lo;
+	ctx->buffer[57] = ctx->lo >> 8;
+	ctx->buffer[58] = ctx->lo >> 16;
+	ctx->buffer[59] = ctx->lo >> 24;
+	ctx->buffer[60] = ctx->hi;
+	ctx->buffer[61] = ctx->hi >> 8;
+	ctx->buffer[62] = ctx->hi >> 16;
+	ctx->buffer[63] = ctx->hi >> 24;
+
+	body(ctx, ctx->buffer, 64);
+
+	result[0] = ctx->a;
+	result[1] = ctx->a >> 8;
+	result[2] = ctx->a >> 16;
+	result[3] = ctx->a >> 24;
+	result[4] = ctx->b;
+	result[5] = ctx->b >> 8;
+	result[6] = ctx->b >> 16;
+	result[7] = ctx->b >> 24;
+	result[8] = ctx->c;
+	result[9] = ctx->c >> 8;
+	result[10] = ctx->c >> 16;
+	result[11] = ctx->c >> 24;
+	result[12] = ctx->d;
+	result[13] = ctx->d >> 8;
+	result[14] = ctx->d >> 16;
+	result[15] = ctx->d >> 24;
+
+	memset(ctx, 0, sizeof(*ctx));
+}
+
+static wxString ComputeMD5(const char *buf, unsigned int len)
+{
+    MD5_CTX ctx;
+    unsigned char signature[16];
+
+    MD5_Init(&ctx);
+    MD5_Update(&ctx, (void *)buf, len);
+    MD5_Final(signature, &ctx);
+
+    wxString result;
+    for (int j = 0; j < (int)sizeof signature; j++)
+    {
+        AppendHex(signature[j], result);
+    }
+
+    return result;
+}
+
 static wxString base64encode(const wxString& str)
 {
 	// Code shamelessly taken from wxWidgets and adopted to encode UTF-8 strings.
@@ -96,39 +435,12 @@
 	{
 		m_handshakeState = http_wait;
 
-#if wxUSE_UNICODE
-		wxWX2MBbuf challenge;
-#else
-		const wxWX2MBbuf challenge;
-#endif
-		int challenge_len;
-		if (user != _T(""))
-		{
-			challenge = base64encode(user + _T(":") + pass).mb_str(wxConvUTF8);
-			challenge_len = strlen(challenge);
-		}
-		else
-		{
-			challenge = (size_t)0;
-			challenge_len = 0;
-		}
-
 		// Bit oversized, but be on the safe side
-		m_pSendBuffer = new char[70 + strlen(host_raw) * 2 + 2*5 + challenge_len + 23];
+		m_pSendBuffer = new char[70 + strlen(host_raw) * 2 + 2*5 + 23];
 
-		if (!challenge)
-		{
-			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nUser-Agent: FileZilla\r\n\r\n",
-				(const char*)host_raw, port,
-				(const char*)host_raw, port);
-		}
-		else
-		{
-			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nProxy-Authorization: Basic %s\r\nUser-Agent: FileZilla\r\n\r\n",
-				(const char*)host_raw, port,
-				(const char*)host_raw, port,
-				(const char*)challenge);
-		}
+		m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nUser-Agent: FileZilla\r\n\r\n",
+			(const char*)host_raw, port,
+			(const char*)host_raw, port);
 
 		m_pRecvBuffer = new char[4096];
 		m_recvBufferLen = 4096;
@@ -302,12 +614,327 @@
 			if (!end)
 				continue;
 
+			unsigned long int nContentLength = 0;
+			bool bUseBasic = false;
+			bool bUseDigest = false;
+			wxString digestStr = _T("");
+			wxString authStr = _T("");
+			wxString header_str(m_pRecvBuffer, wxConvLocal, m_recvBufferPos);
+			wxStringTokenizer tkz(header_str, wxT("\r\n"));
+
+			while (tkz.HasMoreTokens())
+			{
+				wxString token = tkz.GetNextToken();
+				if (token.Mid(0, 15).Lower() == _T("content-length:"))
+				{
+					token.Mid(15).Trim(true).Trim(false).ToULong(&nContentLength);
+					continue;
+				}
+				if (token.Mid(0, 26).Lower() == _T("proxy-authenticate: basic "))
+				{
+					bUseBasic = true;
+					continue;
+				}
+				if (token.Mid(0, 27).Lower() == _T("proxy-authenticate: digest "))
+				{
+					authStr = token.Mid(27);
+					bUseDigest = true;
+					continue;
+				}
+			}
+
+			// read body
+			unsigned long int nBodyLength = 0;
+			char *pBody = new char[nContentLength];
+			while (nBodyLength < nContentLength) {
+				int read;
+
+				do_read = nContentLength - nBodyLength;
+
+				read = m_pSocket->Read(pBody + nBodyLength, do_read, error);
+				if (read == -1)
+				{
+					if (error != EAGAIN)
+					{
+						delete [] pBody;
+						m_proxyState = noconn;
+						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
+						CSocketEventDispatcher::Get().SendEvent(evt);
+					}
+					else
+						m_can_read = false;
+					continue;
+				}
+				if (!read)
+				{
+					delete [] pBody;
+					m_proxyState = noconn;
+					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
+					CSocketEventDispatcher::Get().SendEvent(evt);
+					return;
+				}
+				if (m_pSendBuffer)
+				{
+					delete [] pBody;
+					m_proxyState = noconn;
+					m_pOwner->LogMessage(Debug_Warning, _T("Incoming data before requst fully sent"));
+					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
+					CSocketEventDispatcher::Get().SendEvent(evt);
+					return;
+				}
+				nBodyLength += read;
+			}
+
 			end = strchr(m_pRecvBuffer, '\r');
 			wxASSERT(end);
 			*end = 0;
 			wxString reply(m_pRecvBuffer, wxConvUTF8);
 			m_pOwner->LogMessage(Response, _("Proxy reply: %s"), reply.c_str());
 
+			if (reply.Left(12) == _T("HTTP/1.1 401") || reply.Left(12) == _T("HTTP/1.1 407") ||
+				reply.Left(12) == _T("HTTP/1.0 401") || reply.Left(12) == _T("HTTP/1.0 407"))
+			{
+				if (m_doAuth == 1) {
+					// need to try BASIC when DIGEST failed
+					bUseDigest = false;
+					bUseBasic = true;
+				}
+				if (m_doAuth == 2 || m_user == _T(""))
+				{
+					delete [] pBody;
+					m_pOwner->LogMessage(Debug_Warning, _("already m_doAuth (%d), or m_user is empty (%s)"), m_doAuth, m_user.c_str());
+					m_proxyState = noconn;
+					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNRESET);
+					CSocketEventDispatcher::Get().SendEvent(evt);
+					return;
+				}
+
+				if (bUseDigest)
+				{
+					wxString realm = _T("");
+					wxString nonce = _T("");
+					wxString qop = _T("");
+					wxStringTokenizer tkz2(authStr, wxT(","));
+					while (tkz2.HasMoreTokens())
+					{
+						wxString token2 = tkz2.GetNextToken();
+						wxString name = _T("");
+						wxString value = _T("");
+						wxStringTokenizer tkz3(token2, wxT("="));
+						while (tkz3.HasMoreTokens())
+						{
+							wxString token3 = tkz3.GetNextToken();
+							if (name == _T(""))
+							{
+								name = token3.Lower().Trim(true).Trim(false);
+								continue;
+							}
+							if (token3.Left(1) == _T("\""))
+								value = token3.Mid(1, token3.Len()-2).Trim(true).Trim(false);
+							else
+								value = token3.Trim(true).Trim(false);
+							break;
+						}
+						if (name == _T("realm"))
+							realm = value;
+						else if (name == _T("nonce"))
+							nonce = value;
+						else if (name == _T("qop"))
+							qop = value;
+					}
+					if (realm == _T("") || nonce == _T(""))
+					{
+						delete [] pBody;
+						m_pOwner->LogMessage(Debug_Warning, _T("no realm or nonce"));
+						m_proxyState = conn;
+						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
+						CSocketEventDispatcher::Get().SendEvent(evt);
+						return;
+					}
+					// HA1 = md5(username:realm:password)
+					// if qop = auth or no qop
+					// HA2 = md5(method:digestURI)
+					// if qop = auth-int
+					// HA2 = md5(method:digestURI:md5(body))
+					// if qop = auth or auth-int
+					// response = md5(HA1:nonce:nonceCount:clientNonce:qop:HA2)
+					// if no qop
+					// response = md5(HA1:nonce:HA2)
+					wxString ha1, ha2, response;
+					wxString s;
+					bool bAuth = false;
+					bool bAuthInt = false;
+
+					if (qop != _T(""))
+					{
+						wxStringTokenizer tkz4(qop, wxT(","));
+						while (tkz4.HasMoreTokens())
+						{
+							wxString token4 = tkz4.GetNextToken();
+							if (token4 == _T("auth"))
+								bAuth = true;
+							if (token4 == _T("auth-int"))
+								bAuthInt = true;
+						}
+					}
+
+					s = wxString::Format(_T("%s:%s:%s"), m_user.c_str(), realm.c_str(), m_pass.c_str());
+					const wxWX2MBbuf s_raw = s.mb_str(wxConvUTF8);
+					ha1 = ComputeMD5((const char *)s_raw, strlen(s_raw));
+
+					wxString uri = _T("");
+					if (qop == _T("") || bAuth)
+					{
+						uri = wxString::Format(_T("%s:%u"), m_host.c_str(), m_port);
+						s = wxString::Format(_T("CONNECT:%s"), uri.c_str());
+						const wxWX2MBbuf s_raw = s.mb_str(wxConvUTF8);
+						ha2 = ComputeMD5((const char *)s_raw, strlen(s_raw));
+					}
+					else if (bAuthInt)
+					{
+						uri = wxString::Format(_T("%s:%u"), m_host.c_str(), m_port);
+						wxString bodyMD5 = ComputeMD5((const char *)pBody, nBodyLength);
+						s = wxString::Format(_T("CONNECT:%s:%s"), uri.c_str(), bodyMD5.c_str());
+						const wxWX2MBbuf s_raw = s.mb_str(wxConvLocal);
+						ha2 = ComputeMD5((const char *)s_raw, strlen(s_raw));
+					}
+					else
+					{
+						delete [] pBody;
+						m_pOwner->LogMessage(Debug_Warning, _T("unknown qop: %s"), qop.c_str());
+						m_proxyState = conn;
+						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
+						CSocketEventDispatcher::Get().SendEvent(evt);
+						return;
+					}
+
+					// we don't need body now
+					delete [] pBody;
+					if (bAuth || bAuthInt)
+					{
+						wxString clientNonce = wxString::Format(_T("%04x%04x"), GetRandomNumber(0, 0xffff), GetRandomNumber(0, 0xffff));
+
+						s = wxString::Format(_T("%s:%s:%s:%s:%s:%s"),
+							ha1.c_str(),
+							nonce.c_str(),
+							_T("00000001"),
+							clientNonce.c_str(),
+							bAuth ? _T("auth") : _T("auth-int"),
+							ha2.c_str());
+						const wxWX2MBbuf s_raw = s.mb_str(wxConvLocal);
+						response = ComputeMD5((const char *)s_raw, strlen(s_raw));
+
+						digestStr = wxString::Format(_T("Digest username=\"%s\", realm=\"%s\", nonce=\"%s\", uri=\"%s\", response=\"%s\", qop=%s, nc=00000001, cnonce=\"%s\""),
+							m_user.c_str(),
+							realm.c_str(),
+							nonce.c_str(),
+							uri.c_str(),
+							response.c_str(),
+							bAuth ? _T("auth") : _T("auth-int"),
+							clientNonce.c_str());
+					}
+					else
+					{
+						s = wxString::Format(_T("%s:%s:%s"),
+							ha1.c_str(),
+							nonce.c_str(),
+							ha2.c_str());
+						const wxWX2MBbuf s_raw = s.mb_str(wxConvLocal);
+						response = ComputeMD5((const char *)s_raw, strlen(s_raw));
+
+						digestStr = wxString::Format(_T("Digest username=\"%s\", realm=\"%s\", nonce=\"%s\", uri=\"%s\", response=\"%s\""),
+							m_user.c_str(),
+							realm.c_str(),
+							nonce.c_str(),
+							uri.c_str(),
+							response.c_str());
+					}
+
+					if (bUseBasic)
+						m_doAuth = 1;   // try BASIC later if DIGEST failed
+					else
+						m_doAuth = 2;
+                    			m_recvBufferPos = 0;
+
+					const wxWX2MBbuf host_raw = m_host.mb_str(wxConvUTF8);
+					const wxWX2MBbuf digest_raw = digestStr.mb_str(wxConvLocal);
+
+					int challenge_len = digestStr.Len();
+
+					m_pOwner->LogMessage(Status, _("Sending CONNECT with Digest Authorization"));
+
+                    			// Bit oversized, but be on the safe side
+                    			m_pSendBuffer = new char[70 + m_host.Len() * 2 + 2*5 + challenge_len * 2 + 23 + 40];
+
+					m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nProxy-Authorization: %s\r\nUser-Agent: FileZilla\r\n\r\n",
+						(const char*)host_raw, m_port,
+						(const char*)host_raw, m_port,
+						(const char*)digest_raw);
+
+					wxString host = m_pSocket->GetPeerIP();
+					unsigned int port = m_pSocket->GetRemotePort(error);
+
+					m_pSocket->Close();
+					int res = m_pSocket->Connect(host, port);
+					// Treat success same as EINPROGRESS, we wait for connect notification in any case
+					if (res && res != EINPROGRESS)
+					{
+						m_pOwner->LogMessage(Debug_Warning, _T("connect error?"));
+						m_proxyState = conn;
+						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
+						CSocketEventDispatcher::Get().SendEvent(evt);
+						return;
+					}
+					return;
+				}
+
+				if (bUseBasic)
+				{
+					// we don't need body for this
+					delete [] pBody;
+					m_doAuth = 2;
+                    			m_recvBufferPos = 0;
+
+#if wxUSE_UNICODE
+					wxWX2MBbuf challenge;
+#else
+					const wxWX2MBbuf challenge;
+#endif
+					const wxWX2MBbuf host_raw = m_host.mb_str(wxConvUTF8);
+					int challenge_len;
+
+					challenge = base64encode(m_user + _T(":") + m_pass).mb_str(wxConvUTF8);
+					challenge_len = strlen(challenge);
+
+					m_pOwner->LogMessage(Status, _("Sending CONNECT with Basic Authorization"));
+					// Bit oversized, but be on the safe side
+					m_pSendBuffer = new char[70 + strlen(host_raw) * 2 + 2*5 + challenge_len + 23];
+
+					m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nProxy-Authorization: Basic %s\r\nUser-Agent: FileZilla\r\n\r\n",
+						(const char*)host_raw, m_port,
+						(const char*)host_raw, m_port,
+						(const char*)challenge);
+
+					wxString host = m_pSocket->GetPeerIP();
+					unsigned int port = m_pSocket->GetRemotePort(error);
+
+					m_pSocket->Close();
+					int res = m_pSocket->Connect(host, port);
+					// Treat success same as EINPROGRESS, we wait for connect notification in any case
+					if (res && res != EINPROGRESS)
+					{
+						m_pOwner->LogMessage(Debug_Warning, _T("connect error?"));
+						m_proxyState = conn;
+						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
+						CSocketEventDispatcher::Get().SendEvent(evt);
+						return;
+					}
+					return;
+				}
+				// other authentication method?
+			}
+			delete [] pBody;
+
 			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2"))
 			{
 				m_proxyState = noconn;
diff --strip-trailing-cr -Nur filezilla-3.7.3.orig/src/engine/proxy.h filezilla-3.7.3/src/engine/proxy.h
--- filezilla-3.7.3.orig/src/engine/proxy.h	2012-10-06 11:30:12.000000000 +0800
+++ filezilla-3.7.3/src/engine/proxy.h	2013-10-01 11:59:27.999169200 +0800
@@ -65,6 +65,8 @@
 	int m_recvBufferPos;
 	int m_recvBufferLen;
 
+	int m_doAuth;
+
 	void OnSocketEvent(CSocketEvent& event);
 	void OnReceive();
 	void OnSend();
